@@ -29,6 +29,7 @@
 #include "workspace.h"
 #include "application.h"
 #include "xutils.h"
+#include "pager.h"
 
 G_BEGIN_DECLS
 
@@ -74,9 +75,21 @@ void _wnck_init (void);
 
 #define WNCK_SCREEN_XSCREEN(screen) (_wnck_screen_get_xscreen (screen))
 
-Screen *_wnck_screen_get_xscreen (WnckScreen *screen);
+Screen    *_wnck_screen_get_xscreen    (WnckScreen *screen);
+GdkScreen *_wnck_screen_get_gdk_screen (WnckScreen *screen);
 
 
+void           _wnck_pager_activate_workspace   (WnckWorkspace *wspace);
+int            _wnck_pager_get_n_workspaces     (WnckPager     *pager);
+const char*    _wnck_pager_get_workspace_name   (WnckPager     *pager,
+                                                 int            i);
+WnckWorkspace* _wnck_pager_get_active_workspace (WnckPager     *pager);
+WnckWorkspace* _wnck_pager_get_workspace        (WnckPager     *pager,
+                                                 int            i);
+void           _wnck_pager_get_workspace_rect   (WnckPager     *pager,
+                                                 int            i,
+                                                 GdkRectangle  *rect);
+
 G_END_DECLS
 
 #endif /* WNCK_PRIVATE_H */
