@@ -26,6 +26,7 @@
 #include "workspace.h"
 #include "application.h"
 #include "xutils.h"
+#include "private.h"
 
 /* TODO:
  * 
@@ -59,6 +60,8 @@ typedef struct _WnckTaskClass   WnckTaskClass;
 
 #define TIMEOUT_ACTIVATE 1000
 
+#define N_SCREEN_CONNECTIONS 4
+
 struct _WnckTask
 {
   GObject parent_instance;
@@ -127,6 +130,7 @@ struct _WnckTasklistPrivate
   gint grouping_limit;
 
   guint activate_timeout_id;
+  guint screen_connections [N_SCREEN_CONNECTIONS];
 
   int *size_hints;
   int size_hints_len;
@@ -194,6 +198,9 @@ static gboolean wnck_tasklist_change_active_timeout    (gpointer data);
 static void     wnck_tasklist_activate_task_window     (WnckTask *task);
 
 static void     wnck_tasklist_update_icon_geometries   (WnckTasklist *tasklist);
+static void     wnck_tasklist_connect_screen           (WnckTasklist *tasklist,
+                                                        WnckScreen   *screen);
+static void     wnck_tasklist_disconnect_screen        (WnckTasklist *tasklist);
 
 static gpointer task_parent_class;
 static gpointer tasklist_parent_class;
@@ -410,6 +417,8 @@ wnck_tasklist_finalize (GObject *object)
 
   tasklist = WNCK_TASKLIST (object);
 
+  wnck_tasklist_disconnect_screen (tasklist);
+
   /* Tasks should have gone away due to removing their
    * buttons in container destruction
    */
@@ -604,12 +613,12 @@ wnck_tasklist_score_groups (WnckTasklist *tasklist,
 
 	  if (first_name == NULL)
 	    {
-	      first_name = wnck_window_get_name (win_task->window);
+	      first_name = wnck_window_get_icon_name (win_task->window);
 	      n_same_title++;
 	    }
 	  else
 	    {
-	      if (strcmp (wnck_window_get_name (win_task->window),
+	      if (strcmp (wnck_window_get_icon_name (win_task->window),
 			  first_name) == 0)
 		n_same_title++;
 	    }
@@ -1051,35 +1060,31 @@ wnck_tasklist_remove (GtkContainer   *container,
     }
 }
 
-
-GtkWidget*
-wnck_tasklist_new (WnckScreen *screen)
+static void
+wnck_tasklist_connect_screen (WnckTasklist *tasklist,
+			      WnckScreen   *screen)
 {
-  WnckTasklist *tasklist;
-  GList        *windows;
+  GList *windows;
+  guint *c;
+  int    i;
 
-  tasklist = g_object_new (WNCK_TYPE_TASKLIST, NULL);
+  i = 0;
+  c = tasklist->priv->screen_connections;
 
-  tasklist->priv->screen = screen;
-  
-  tasklist->priv->tooltips = gtk_tooltips_new ();
-  gtk_object_ref (GTK_OBJECT (tasklist->priv->tooltips));
-  gtk_object_sink (GTK_OBJECT (tasklist->priv->tooltips));
+  c [i++] = g_signal_connect_object (G_OBJECT (screen), "active_window_changed",
+                                     G_CALLBACK (wnck_tasklist_active_window_changed),
+                                     tasklist, 0);
+  c [i++] = g_signal_connect_object (G_OBJECT (screen), "active_workspace_changed",
+                                     G_CALLBACK (wnck_tasklist_active_workspace_changed),
+                                     tasklist, 0);
+  c [i++] = g_signal_connect_object (G_OBJECT (screen), "window_opened",
+                                     G_CALLBACK (wnck_tasklist_window_added),
+                                     tasklist, 0);
+  c [i++] = g_signal_connect_object (G_OBJECT (screen), "window_closed",
+                                     G_CALLBACK (wnck_tasklist_window_removed),
+                                     tasklist, 0);
 
-  wnck_tasklist_update_lists (tasklist);
-  
-  g_signal_connect_object (G_OBJECT (screen), "active_window_changed",
-			   G_CALLBACK (wnck_tasklist_active_window_changed),
-			   tasklist, 0);
-  g_signal_connect_object (G_OBJECT (screen), "active_workspace_changed",
-			   G_CALLBACK (wnck_tasklist_active_workspace_changed),
-			   tasklist, 0);
-  g_signal_connect_object (G_OBJECT (screen), "window_opened",
-			   G_CALLBACK (wnck_tasklist_window_added),
-			   tasklist, 0);
-  g_signal_connect_object (G_OBJECT (screen), "window_closed",
-			   G_CALLBACK (wnck_tasklist_window_removed),
-			   tasklist, 0);
+  g_assert (i == N_SCREEN_CONNECTIONS);
 
   windows = wnck_screen_get_windows (screen);
   while (windows != NULL)
@@ -1087,6 +1092,55 @@ wnck_tasklist_new (WnckScreen *screen)
       wnck_tasklist_connect_window (tasklist, windows->data);
       windows = windows->next;
     }
+}
+
+static void
+wnck_tasklist_disconnect_screen (WnckTasklist *tasklist)
+{
+  int i;
+
+  i = 0;
+  while (i < N_SCREEN_CONNECTIONS)
+    {
+      if (tasklist->priv->screen_connections [i] != 0)
+        g_signal_handler_disconnect (G_OBJECT (tasklist->priv->screen),
+                                     tasklist->priv->screen_connections [i]);
+
+      tasklist->priv->screen_connections [i] = 0;
+
+      ++i;
+    }
+}
+
+void
+wnck_tasklist_set_screen (WnckTasklist *tasklist,
+			  WnckScreen   *screen)
+{
+  if (tasklist->priv->screen == screen)
+    return;
+
+  if (tasklist->priv->screen)
+    wnck_tasklist_disconnect_screen (tasklist);
+
+  tasklist->priv->screen = screen;
+
+  wnck_tasklist_update_lists (tasklist);
+
+  wnck_tasklist_connect_screen (tasklist, screen);
+}
+
+GtkWidget*
+wnck_tasklist_new (WnckScreen *screen)
+{
+  WnckTasklist *tasklist;
+
+  tasklist = g_object_new (WNCK_TYPE_TASKLIST, NULL);
+
+  tasklist->priv->tooltips = gtk_tooltips_new ();
+  gtk_object_ref (GTK_OBJECT (tasklist->priv->tooltips));
+  gtk_object_sink (GTK_OBJECT (tasklist->priv->tooltips));
+
+  wnck_tasklist_set_screen (tasklist, screen);
 
   return GTK_WIDGET (tasklist);
 }
@@ -1548,6 +1602,9 @@ wnck_task_popup_menu (WnckTask *task,
       
       l = l->next;
     }
+
+  gtk_menu_set_screen (GTK_MENU (menu),
+		       _wnck_screen_get_gdk_screen (task->tasklist->priv->screen));
   
   gtk_widget_show (menu);
   gtk_menu_popup (GTK_MENU (menu),
@@ -1597,13 +1654,13 @@ wnck_task_get_text (WnckTask *task)
   if (task->is_application)
     {
       return g_strdup_printf ("%s (%d)",
-                              wnck_application_get_name (task->application),
+                              wnck_application_get_icon_name (task->application),
                               g_list_length (task->windows));
     }
   else
     {
       state = wnck_window_get_state (task->window);
-      name = wnck_window_get_name (task->window);
+      name = wnck_window_get_icon_name (task->window);
       
       if (state & WNCK_WINDOW_STATE_MINIMIZED)
 	return g_strdup_printf ("[%s]", name);
@@ -1878,6 +1935,9 @@ wnck_task_button_press_event (GtkWidget	      *widget,
       g_object_add_weak_pointer (G_OBJECT (task->action_menu),
                                  (void**) &task->action_menu);
       
+      gtk_menu_set_screen (GTK_MENU (task->action_menu),
+                           _wnck_screen_get_gdk_screen (task->tasklist->priv->screen));
+
       gtk_widget_show (task->action_menu);
       gtk_menu_popup (GTK_MENU (task->action_menu),
                       NULL, NULL,
