@@ -40,7 +40,8 @@ typedef enum
   WNCK_WINDOW_STATE_SHADED                 = 1 << 3,
   WNCK_WINDOW_STATE_SKIP_PAGER             = 1 << 4,
   WNCK_WINDOW_STATE_SKIP_TASKLIST          = 1 << 5,
-  WNCK_WINDOW_STATE_STICKY                 = 1 << 6
+  WNCK_WINDOW_STATE_STICKY                 = 1 << 6,
+  WNCK_WINDOW_STATE_HIDDEN                 = 1 << 7
 } WnckWindowState;
 
 typedef enum
