@@ -959,6 +959,30 @@ _wnck_activate_workspace (Screen *screen,
 	      &xev);
 }
 
+void
+_wnck_toggle_showing_desktop (Screen  *screen,
+                              gboolean show)
+{
+  XEvent xev;
+  
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = DisplayOfScreen (screen);
+  xev.xclient.window = RootWindowOfScreen (screen);
+  xev.xclient.message_type = _wnck_atom_get ("_NET_SHOWING_DESKTOP");
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = show != FALSE;
+  xev.xclient.data.l[1] = 0;
+  xev.xclient.data.l[2] = 0;
+
+  XSendEvent (DisplayOfScreen (screen),
+	      RootWindowOfScreen (screen),
+              False,
+	      SubstructureRedirectMask | SubstructureNotifyMask,
+	      &xev);
+}
+
 Window
 _wnck_get_group_leader (Window xwindow)
 {
@@ -2173,15 +2197,22 @@ _wnck_set_dock_type_hint (Window xwindow)
   _wnck_error_trap_pop ();
 }
 
+/* orientation of pager */
 #define _NET_WM_ORIENTATION_HORZ 0
 #define _NET_WM_ORIENTATION_VERT 1
 
+/* starting corner for counting spaces */
+#define _NET_WM_TOPLEFT     0
+#define _NET_WM_TOPRIGHT    1
+#define _NET_WM_BOTTOMRIGHT 2
+#define _NET_WM_BOTTOMLEFT  3
+
 void
 _wnck_set_desktop_layout (Screen *xscreen,
                           int     rows,
                           int     columns)
 {
-  gulong data[3];
+  gulong data[4];
 
   /* FIXME: hack, hack, hack so as not
    * to have to add a orientation param
@@ -2192,8 +2223,9 @@ _wnck_set_desktop_layout (Screen *xscreen,
   g_assert ((rows == 0) || (columns == 0));
 
   data[0] = (columns == 0) ? _NET_WM_ORIENTATION_HORZ : _NET_WM_ORIENTATION_VERT;
-  data[1] = rows;
-  data[2] = columns;
+  data[1] = columns;
+  data[2] = rows;
+  data[3] = _NET_WM_TOPLEFT;
   
   _wnck_error_trap_push ();
 
@@ -2201,7 +2233,7 @@ _wnck_set_desktop_layout (Screen *xscreen,
                    RootWindowOfScreen (xscreen),
 		   _wnck_atom_get ("_NET_DESKTOP_LAYOUT"),
 		   XA_CARDINAL, 32, PropModeReplace,
-		   (guchar *)&data, 3);
+		   (guchar *)&data, sizeof (data));
 
   _wnck_error_trap_pop ();
 }
@@ -2416,3 +2448,51 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
 
   return FALSE;
 }
+
+/* stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org> */
+typedef struct
+{
+  char *stock_id;
+  const guint8 *icon_data;
+} StockIcon;
+
+void
+_wnck_stock_icons_init (void)
+{
+  GtkIconFactory *factory;
+  int i;
+  static gboolean done = FALSE;
+
+  StockIcon items[] =
+  {
+    { WNCK_STOCK_DELETE,   stock_delete_data   },
+    { WNCK_STOCK_MINIMIZE, stock_minimize_data },
+    { WNCK_STOCK_MAXIMIZE, stock_maximize_data }
+  };
+
+  if (done)
+    return;
+
+  done = TRUE;
+  
+  factory = gtk_icon_factory_new ();
+  gtk_icon_factory_add_default (factory);
+
+  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
+    {
+      GtkIconSet *icon_set;
+      GdkPixbuf *pixbuf;
+
+      pixbuf = gdk_pixbuf_new_from_inline (-1, items[i].icon_data,
+					   FALSE,
+					   NULL);
+
+      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
+      gtk_icon_set_unref (icon_set);
+		
+      g_object_unref (G_OBJECT (pixbuf));
+    }
+
+  g_object_unref (G_OBJECT (factory));
+}
